# Chapter 3. Types, Values, and Variables

## 3.11 Summary

Keypoints from this chapter:
* How to write and manipulate numbers and strings of text in JavaScript.
* How to work with JavaScript’s other primitive types: booleans, Symbols, null, and undefined.
* The differences between immutable primitive types and mutable reference types.
* How JavaScript converts values implicitly from one type to another and how you can do so explicitly in your programs.
* How to declare and initialize constants and variables (including with destructuring assignment) and the lexical scope of the variables and constants you declare.

## 3.1 Overview and Definitions

JS types has 2 categories of data types:
* **primitive** types: numeric, string, boolean, special JS values (`null` & `undefined`)
* **object** types: any JS values that not primitive types.
  * It's a collection of properties where each property has a name and a value (primitive or object)
  * Ordinary JS object is unordered collection of named values.
  * Special JS object like 
    * **Array** defined in JS is an ordered collection of numbered values.
    * **Map**: mapping from keys to values
    * **Set**: set of values
    * **"typed array" types** for operations on array of bytes/binary
    * **RegExp** for textual patterns
    * **Date** type for date & times
    * **Error & subtypes** for errors when executing JS code.

Functional Programming of JS:
* Functions/Classes are not just part of language syntax, but they are also values that can be manipulated by JS

Garbage collection of JS:
* JS interpreter automatically collect garbage for memory management.

OOP of JS:
* type themselves define methods for working with values. e.g. `a.sort()`

Mutable/Immutable of JS:
* Primitive types are **immutable**: number, booleans
* Object types are **immutable**: change array

Value convertion of JS:
* JS convert values automatically (e.g. from number to string)
* Certain rules are there.

Constant & Variable declaration:
* Constant & Variables allow programmer to refer values using names.
* Declare constant using `const`
* Declare variable using `let` (for new JS), or `var` for old JS version
* Constant & Variables are *untyped* (i.e. declaration do no specify type of assigned values)

## 3.2 Numbers

Number:
* JS's primitive numeric type
* Represent *Integers* & *Float*
* Using 64-bit floating point format
  * large as +/- 1.79769x10^308
  * Small as +/- 5 x 10^-324

When a number appears directly in JS code, it's **numerical literal**

### 3.2.1 Integer Literals

* Base-10 integer: `0`, `3`, `100000`
* Hexadecimal (base-16) values: `0xff`, `0xBADCAFE`
* Binary (base 2) & Octal (base 8) after ES6: `0b10101`, `0o377`

### 3.2.2 Floating-Point Literals

* Traditional notation: `3.14`, `2345.6789`
* Exponential notation: `6.02e23` means `6.02 x 10^23`

#### Separations in numeric literals

Future JS format will recognize `_` within numeric literals for breaking long literals

e.g.:
* `let billion = 1_000_000_000;`
* `let bytes = 0x89_AB_CD_EF;`

### 3.2.3 Arithmetic in JS

Basic Arithmetic operators:
* addition: `+`
* subtraction: `-`
* multiplication: `*`
* division: `/`
* modulo: `%`
* exponentiation: `**`

`Math` object has more complex math operations, check reference for details:
* `Math.pow(2,53)`
* `Math.round(.6)`
* ...

Note:
* Arithmetic in JS do not raise errors for *overflow*/*underflow*/*division by 0*
* Overflow (result is larger than largest representable number) -> `Infinity`
* Underflow (result is smaller than smallest representable) -> `-Inifinity`
* Division by 0 return either `Infinity` or `-Inifinity`
* `0/0` return `NaN` (**Not a Number**) as the value is not well-defined
* `Infinity/Infinity` return `NaN`

JS predefined a series of global constants:
* `Infinity`
* `Number.POSITIVE_INFINITY`
* `1/0`
* ...
check reference for all global constant information

* `NaN` cannot compare to other value (others can)
* Solution to the problem: `x != x` or `Number.isNaN(x)` for comparison

### 3.2.4 Binary Floating-Point and Rounding Errors

Real number in real life is continuous & infinite. But due to computer is binary, only finite number of them can be represented by JS floating-point format. Hence, in JS, real number are approximation of the actual number.

JS us IEEE-754 floating-point representation (a binary representation, that can only represent 1/2, 1/7, 1/1024). Number like `.1` (i.e. `1/10`) can only be approximated

This problem is generic across computing language using binary representation. Solution include: **manipulate monetary values as integer cents rather than fractional dollars**

### 3.2.5 Arbitrary Precision Integers with BigInt

New numeric type **BigInt** (new features of JS) defined in ES2020.

```js
1234n         // not-so-big BigInt
0b1111n       // Binary BigInt
0o7777n       // Octal BigInt
0x800000000n
```

BigInt: 
* a numeric type for large **(64-bit) integer**
* BigInt literals are shown above. (i.e. digits followed by `n`)
* Can convert from other number using `BigInt()`

```js
BigInt(Number.MAX_SAFE_INTEGEG) // => 900719925474099n
let string = "1" + "0".repeat(100); // 1 followed by 100 zeros
BigInt(string)
```

Arithmetic operations with BigInt:

```js
1000n + 2000n
3000n - 2000n
2000n * 3000n
3000n / 997n
...
```

### 3.2.6 Dates and Times

JS has simple Date class, the timestamp count since Jan 1, 1970

```js
let timestamp = Date.now();
let now = new Date();
let ms = new.getTime(); // milisecond timestamp
let iso = now.toISOString(); // Convert to a string in standard format.
```

## 3.3 Text

* JS use *string* to represent text.
* *string* = immutable ordered sequence of 16-bit values.
* JS use **zero-based indexing** for string & array. i.e. 1st value start from 0
  * empty string has length 0

JS strings & codepoints & characters
* JS uses UTF-16 encoding for characters.
  * Most frequently used characters have 16bit **codepoints**
* Some Unicode characters cannot fit in 16bits, may be represented as a sequence of 2 16-bit values (i.e. **surrogate pair**) as shown below

```js
let euro = "€";
let love = "❤";
euro.length   // => 1: this character has one 16-bit element
love.length   // => 2: UTF-16 encoding of ❤ is "\ud83d\udc99"
```

* Pre ES6, looping through surrogate pair will treat each 16bit as independent value
* In ES6, strings are *iterable*, so iterator will recognize these surrogate pairs

### 3.3.1 String Literals

Pre ES6, Coding string literals in JS: cover literals within single/double quotes
* double/single quotes can be contained within single/double quotes

```js
""  // The empty string: it has zero characters
'testing' // single quotes
"3.14" // double quotes
'name="myform"' // double quotes contained within string delimited by single-quote
"Wouldn't you prefer O'Reilly's book?" // 
"τ is the ratio of a circle's circumference to its radius"
```

In ES6, backticks is added to contain string literals, which allow JS expressions to be embedded within string literals

```js
`"She said 'hi'", he said.`
```

#### multi-line string and new line character sequence

Pre ES5: JS required string literals to be coded in single line.
* Common practice: String literals are broken into multi-lines by concatenated with `+`

In ES5: JS can break string into multilines by **backslash** `\`
```js
// A one-line string written on 3 lines:
"one\
 long\
 line"
```

In ES5: JS can create a two line string written in one line code using **new lien character sequence** `\n`
```js
// A string representing 2 lines written on one line:
'two\nlines'
```

In ES6: backtick allows strings to be broken into multiple lines, while line terminator are part of the string literal:
```js
// A two-line string written on two lines:
`the newline character at the end of this line
is included literally in this string`
```

#### JS & HTML mixed up

JS programming may contain strings of HTML code, while HTML code may contains strings of JS. Both JS & HTML use double/single quote for strings.

Common practice: use one style of quotes for JS and other style for HTML.

```html
<button onclick="alert('Thank you')">Click Me</button>
```
* string "Thank you" is single-quoted within a JS expression
* JS expression is then double-quoted within HTML event-handler attribute

### 3.3.2 Escape Sequences in String Literals

Backslash character `\` combined with specific character = **excape sequence**, which represent a special character

JS escape sequences
| Sequence | Character represented                                                                                                   |
| -------- | ----------------------------------------------------------------------------------------------------------------------- |
| `\0`     | NULL (\u0000)                                                                                                           |
| `\b`     | Backspace (\u0008)                                                                                                      |
| `\t`     | Horizontal tab (\u0009)                                                                                                 |
| `\n`     | Newline (\u000A)                                                                                                        |
| `\v`     | Vertical tab (\u000B)                                                                                                   |
| `\f`     | Form feed (\u000C)                                                                                                      |
| `\r`     | Carriage return (\u000D)                                                                                                |
| `\"`     | Double quote (\u0022)                                                                                                   |
| `\'`     | Apostrophe or single quote (\u0027)                                                                                     |
| `\\`     | Backslash (\u005C)                                                                                                      |
| `\xnn`   | The Unicode character specified by the two hexadecimal digits nn                                                        |
| `\unnnn` | The Unicode character specified by the four hexadecimal digits nnnn                                                     |
| `\u{n}`  | The Unicode character specified by the codepoint n, where n is one to six hexadecimal digits between 0 and 10FFFF (ES6) |

### 3.3.3 Working with Strings

* concatenate strings (JS' built-in feature) using `+`:
```js
let msg = "Hello, " + "world";   // Produces the string "Hello, world"
let greeting = "Welcome to my blog," + " " + name;
```

* String comparison using `===` (equal) and `!==` (inequal). 
  * Comparison is done by checking 16-bit values
* Checking length of a string via length property of the string `.length`

Other than methods/property mentioned above, JS has rich API (methods):

```js
let s = "Hello, world"; // Start with some text.

// Obtaining portions of a string
s.substring(1,4)        // => "ell": the 2nd, 3rd, and 4th characters.
s.slice(1,4)            // => "ell": same thing
s.slice(-3)             // => "rld": last 3 characters
s.split(", ")           // => ["Hello", "world"]: split at delimiter string

// Searching a string
s.indexOf("l")          // => 2: position of first letter l
s.indexOf("l", 3)       // => 3: position of first "l" at or after 3
s.indexOf("zz")         // => -1: s does not include the substring "zz"
s.lastIndexOf("l")      // => 10: position of last letter l

// Boolean searching functions in ES6 and later
s.startsWith("Hell")    // => true: the string starts with these
s.endsWith("!")         // => false: s does not end with that
s.includes("or")        // => true: s includes substring "or"

// Creating modified versions of a string
s.replace("llo", "ya")  // => "Heya, world"
s.toLowerCase()         // => "hello, world"
s.toUpperCase()         // => "HELLO, WORLD"
s.normalize()           // Unicode NFC normalization: ES6
s.normalize("NFD")      // NFD normalization. Also "NFKC", "NFKD"

// Inspecting individual (16-bit) characters of a string
s.charAt(0)             // => "H": the first character
s.charAt(s.length-1)    // => "d": the last character
s.charCodeAt(0)         // => 72: 16-bit number at the specified position
s.codePointAt(0)        // => 72: ES6, works for codepoints > 16 bits

// String padding functions in ES2017
"x".padStart(3)         // => "  x": add spaces on the left to a length of 3
"x".padEnd(3)           // => "x  ": add spaces on the right to a length of 3
"x".padStart(3, "*")    // => "**x": add stars on the left to a length of 3
"x".padEnd(3, "-")      // => "x--": add dashes on the right to a length of 3

// Space trimming functions. trim() is ES5; others ES2019
" test ".trim()         // => "test": remove spaces at start and end
" test ".trimStart()    // => "test ": remove spaces on left. Also trimLeft
" test ".trimEnd()      // => " test": remove spaces at right. Also trimRight

// Miscellaneous string methods
s.concat("!")           // => "Hello, world!": just use + operator instead
"<>".repeat(5)          // => "<><><><><>": concatenate n copies. ES6
```

* strings are immutable in JS. Methods shown above return new strings, instead of modifying original string
* strings can be read as array, `s[s.slength-1]` return 2nd last char

### 3.3.4 Template Literals

In ES6, backtickes are added:
```js
let s = `hello world`;
```

**Template literals** = string literal created using backticks
* it can include JS expression.
* its final value is computed by evaluating expression, convert these values into strings, combining to get result
  * JS expression: `${ expression }`
  * Everything outside `${}` is normal string literal text.

```js
let name = "Bill";
let greeting = `Hello ${ name }.`;  // greeting == "Hello Bill."
```

* Template literal can include multiple expressions.
  * Use any escape char
  * Span multi-lines

```js
let errorMessage = `\ // backslash escape the initial newline
\u2718 Test failure at ${filename}:${linenumber}:
${exception.message}
Stack trace:
${exception.stack}
`;
```

#### Tagged template literals

**Tagged Template Literals**: Template literals's uncommonly used feature
* If a function name (also called "tag function") comes right before the opening backtick, (e.g. shown below)
  * then the text and values of the expressions within the template literal are passed to the function
  * And value of this **Tagged Template Literals** is the return value of the function
中文翻译：
* 如果 function name 在 opening backtick 之前
  * 那 backtick 以内的文字和值都会被直接传递给 function
  * 这个方程返回的值就是 **Tagged Template Literals** 的值

```js
`\n`.length // => 1: the string has a single newline character
String.raw`\n`.length // => 2: a backslash character and the letter n

// 这里 `\n` 作为 tagged template literals 被传递给了方程，而导致产生的结果不一样
```

Usage of Tagged Template Literals:
* Applying HTML or SQL escaping to the values before substuting them into the text
* etc.

We can define our own template tag function

### 3.3.5 Pattern Matching

* **RegExp (Regular Expression)**: a datatype defined by JS. 
  * Used for describing/matching string patterns.
* RegExp is not fundamental datatype in JS
* Grammer & API of RegExp is complex.

Grammer of RegExp:
  * RegExp literal = Text btw a pair of slashes
  * one or more letter can follow 2nd slash to change meaning of pattern
```js
/^HTML/;            // Match the letters HTML at the start of a string
/[1-9][0-9]*/;      // Match a nonzero digit, followed by any number of digits
/\bjavascript\b/i;  // Match a "javascript" as a work, case-sensitive
```

API of RegExp datatype (e.g.):
```js
let text = "testing: 1, 2, 3";   // Sample text
let pattern = /\d+/g;            // Matches all instances of one or more digits
pattern.test(text)               // => true: a match exists
text.search(pattern)             // => 9: position of first match
text.match(pattern)              // => ["1", "2", "3"]: array of all matches
text.replace(pattern, "#")       // => "testing: #, #, #"
text.split(/\D+/)                // => ["","1","2","3"]: split on nondigits
```

## 3.4 Boolean Values

* **Boolean values**: evaluated using reserved words `true` & `false`
* Comparison in JS: 
  * Equal to: `===`. e.g. `a === 4`
  * Not-Equal to: `!==`
* **Any** JS value can be converted to a boolean value:
  * Following (i.e. **falsy values**) are converted to false.
  * All values aren't falsy values are **truthy**
```js
undefined
null
0
-0
NaN
""  // the empty string
```

truthy values can be proven as shown:
```js
if (o!== null)...
```

Methods of Boolean values:
* `toString()`: convert `true` to "true".

3 Important boolean operator:
* `&&`: Boolean AND operator. It evaluate (generate) a truthy value iff both operands are truthy
* `||`: Boolean OR operator.
* `!`: Boolean NOT operator

```js
if ((x === 0 && y === 0) || !(z === 0)) {
    // x and y are both zero or z is non-zero
}
```

## 3.5 null and undefined

`null` 
* Def: special value, indicating "absence of a value"
* `typeof(null)` return `"object"`; So it means `null` is a special object, with value "no object".
* `null` is the sole member of this own type.
  * It indicate "no value" for data type like number, strings and objects.
* similar in other language: `NULL` in C++

`undefined`
* Def: a deeper kind of absence, it's the value of variables that have not been initialized
  * When query the value of an object project property or array element that **does not exist or not initilized**, return `undefined`
* It's predefined global constant, that's initilzed with value `undefined`
* `typeof(undefined)` return `"undefined"`, hence it's a special type

`null` similar as `undefined`
* both can use interchangeably.
* **equality operator `==`** will evalute them to be equal; **strict equality operator`===`** can distringuish them
* Both are falsy values
* Both of them have no property/methods
* visiting property/methods using `.` or `[]` returns `TypeError`

`null` different from `undefined` (by author)
* `undefined`: a system-level, unexpected or error like absence of value
* `null`: program-level, normal, or expected absence of value.
  * Prefer to be assigned to variable or passed as return

## 3.6 Symbols

Prerequisite knowledge: JS's Object type is an unordered collection of properties, each property is a name/value pair.

Before ES6, property name are strings. Since ES6, **symbols** can also be used as **property name of an object**.

```js
let strname = "string name";      // A string to use as a property name
let symname = Symbol("propname"); // A Symbol to use as a property name
typeof strname                    // => "string": strname is a string
typeof symname                    // => "symbol": symname is a symbol
let o = {};                       // Create a new object
o[strname] = 1;                   // Define a property with a string name
o[symname] = 2;                   // Define a property with a Symbol name
o[strname]                        // => 1: access the string-named property
o[symname]                        // => 2: access the symbol-named property
```

How to obtain a Symbol value:
* Symbol type does not have a literal syntax. (e.g. char used for string literal syntax)
* Use `Symbol()` function to obtain Symbol value as shown above.
  * Syntax e.g.: `let variable_name = Symbol("property_name")`

How to use Symbol value:
* Use a symbol value as a property name to add a new property to an object.
* `Symbol()` function never return same (symbol) value, even given same string (`"property_name"`). So no worry to overwrite original property in Object.

Symbol value related functions:
* `Symbol()`: used to create symbol value
* `s.toString()`: return string used as argument when creating the value
  * Note: two different symbol value can have same string
* `Symbol.for()`: used to search symbol value with given string
  * JS defines a global Symbol registry
  * If no Symbol are associated with that string, a new one is created&returned.

```js
let s = Symbol("sym_x");
s.toString()             // => "Symbol(sym_x)"
```

```js
let s = Symbol.for("shared");
let t = Symbol.for("shared");
s === t          // => true
s.toString()     // => "Symbol(shared)"
Symbol.keyFor(t) // => "shared"
```

## 3.7 The Global Object

**Global Object**: properties of this JS object are globally defined identifiers, and are available to a JS program.
* Usage: When JS interpreter starts (e.g. Node or new page in web browser), it creates a new global object and initialize its properties:
  * Global constants: e.g. `undefined`, `Infinity`, and `NaN`
  * Global functions: e.g. `isNaN()`, `parseInt()` (§3.9.2), and `eval()` (§4.12)
  * Constructor functions: e.g. `Date()`, `RegExp()`, `String()`, `Object()`, and `Array()` (§3.9.2)
  * Global objects: e.g. `Math` and `JSON` (§6.8)
* Initialized properties can be viewed as reserved words (尽管不是).

Global object in Node:
* global object has property with name `global`.
* In Node, we can refer the global object using the name `global` 

Global obj in web browsers:
* **Window object** act as a global obj for all JS code in the browser window.
* Use `window` property of this global obj to refer it.

Since ES2020, `globalThis` is used to refer to the global obj.

## 3.8 Immutable Primitive Values and Mutable Object References

Summary: in JS, primitive values are immutable, object values are mutable

Primitives are immutable:
* Only new values can be created/returned
* Can be compared by value:
  * Two string/array are equal iif they have same lenght, and char at index is the same.
```js
let s = "hello";   // Start with some lowercase text
s.toUpperCase();   // Returns "HELLO", but doesn't alter s
s                  // => "hello": the original string has not changed
```

Objects are mutable:
* Their values can change
* Not compared by value. Two distinct/independent obj are not equal even if they have the same properties and values.
* They are also called **reference types** to be distinguished from JS's primitive types.
  * Two obj values are the same iff they **refer** to the same underlying object. e.g. `let b = a`

Mutable obj
```js
let o = { x: 1 };  // Start with an object
o.x = 2;           // Mutate it by changing the value of a property
o.y = 3;           // Mutate it again by adding a new property

let a = [1,2,3];   // Arrays are also mutable
a[0] = 0;          // Change the value of an array element
a[3] = 4;          // Add a new array element
```

Two distinct/independent obj
```js
let o = {x: 1}, p = {x: 1};  // Two objects with the same properties
o === p                      // => false: distinct objects are never equal
let a = [], b = [];          // Two distinct, empty arrays
a === b                      // => false: distinct arrays are never equal
```

Obj are same if they refer the same obj:
```js
let a = [];   // The variable a refers to an empty array.
let b = a;    // Now b refers to the same array.
b[0] = 1;     // Mutate the array referred to by variable b.
a[0]          // => 1: the change is also visible through variable a.
a === b       // => true: a and b refer to the same object, so they are equal.
```

How to compare 2 distinct objs:
* Compare their properties/elements one by one

```js
function equalArrays(a, b) {
    if (a === b) return true;                // Identical arrays are equal
    if (a.length !== b.length) return false; // Different-size arrays not equal
    for(let i = 0; i < a.length; i++) {      // Loop through all elements
        if (a[i] !== b[i]) return false;     // If any differ, arrays not equal
    }
    return true;                             // Otherwise they are equal
}
```

## 3.9 Type Conversions

**Implicit Conversion**: JS can convert values to its required data type automatically

```js
10 + " objects"     // => "10 objects":  Number 10 converts to a string
"7" * "4"           // => 28: both strings convert to numbers
let n = 1 - "x";    // n == NaN; string "x" can't convert to a number
n + " objects"      // => "NaN objects": NaN converts to string "NaN"
```

* Primitive-to-primitive conversion: Refer to [JS primitive-to-primitive conversion table](https://riptutorial.com/javascript/example/16970/primitive-to-primitive-conversion-table) for details
* Object-to-primitive conversion: some more complicated

### 3.9.1 Conversions and Equality

Two equality operators used JS to test whether 2 values are equal:
* **strict equality operator**: `===`, two operands are not equal if they are not of same type
* **flexible equality operator**: `==`, JS can convert types

```js
null == undefined // => true: These two values are treated as equal.
"0" == 0          // => true: String converts to a number before comparing.
0 == false        // => true: Boolean converts to number before comparing.
"0" == false      // => true: Both operands convert to 0 before comparing!
```

§4.9.1 explain conversion performed by `==` operator

### 3.9.2 Explicit Conversions

Explicit conversion is needed for customized requirements.

#### Simplist explicit conversion

Using functions:
* `Boolean()`
* `Number()`
* `Boolean()`

```js
Number("3")    // => 3
String(false)  // => "false":  Or use false.toString()
Boolean([])    // => true
```

Using `.toString()` method:
* All value (except `null` & `undefined`) has `toString()` method.
* Result is as same as `String()`

Legacy from the past: `Boolean`, and other functions can be used as constructor with `new`. It create **wrapper** object that behaves just like a primitive. Don't use it

#### Use some JS operators

* `+` or `-`: if one operand is string, it convert another to string
* `!`: first convert operand to a boolean, then negate/revert it.

```js
x + ""   // => String(x)
+x       // => Number(x)
x-0      // => Number(x)
!!x      // => Boolean(x): Note double !
```

#### Formatting & Parsing number

Convert Number class object to string using `toString()`:
* which accept optional argument for radix (基数)

```js
let n = 17;
let binary = "0b" + n.toString(2);  // binary == "0b10001"
let octal = "0o" + n.toString(8);   // octal == "0o21"
let hex = "0x" + n.toString(16);    // hex == "0x11"
```

Convert Number class object to string, with control over number of decimal places:
* `.toFixed()`: string with specified number of digits after the decimal point.
* `.toExponential()`: string in exponential notation; given specified number of digits after decimal point.
* `.toPrecision()`: string with number of significant digits you specify. (即有几个可见数位)

```js
let n = 123456.789;
n.toFixed(0)         // => "123457"
n.toFixed(2)         // => "123456.79"
n.toFixed(5)         // => "123456.78900"
n.toExponential(1)   // => "1.2e+5"
n.toExponential(3)   // => "1.235e+5"
n.toPrecision(4)     // => "1.235e+5"
n.toPrecision(7)     // => "123456.8"
n.toPrecision(10)    // => "123456.7890"
```

#### Parsing string to number

Problem of `Number()`:
* Only accept integer or floating-point
* Only worked for base-10 integers
* Does not allow trailing characters (string). e.g. `10blahblah`

Solution:
* `parseInt()`
  * global function; parse only integers;
  * if begin with "0x" or "0X", interpret it as hex
  * Accept optional 2nd argument specifying radix
* `parseFloat()`
  * global function; parse both integer and floating-point numbers;

Both `parseInt()` & `parseFloat()`:
* skip leading whitespace
* parse as many numeric characters as they can
* Ignore all following char

```js
parseInt("3 blind mice")     // => 3
parseFloat(" 3.14 meters")   // => 3.14
parseInt("-12.34")           // => -12
parseInt("0xFF")             // => 255
parseInt("0xff")             // => 255
parseInt("-0XFF")            // => -255
parseFloat(".1")             // => 0.1
parseInt("0.1")              // => 0
parseInt(".1")               // => NaN: integers can't start with "."
parseFloat("$72.47")         // => NaN: numbers can't start with "$"

parseInt("11", 2)     // => 3: (1*2 + 1)
parseInt("ff", 16)    // => 255: (15*16 + 15)
parseInt("zz", 36)    // => 1295: (35*36 + 35)
parseInt("077", 8)    // => 63: (7*8 + 7)
parseInt("077", 10)   // => 77: (7*10 + 7)
```

### 3.9.3 Object to Primitive Conversions

JS has complicated rule to convert objects to primitive values.

TODO: finish this section

## 3.10 Variable Declaration and Assignment

Binding a name to a value = "assigning a value to a *variable*", variable's value can change as program runs.

Steps of using variable in JS:
1. **declare** variable using `let` `const` keywords (in ES6)
2. Use declared/initialized variable

### 3.10.1 Declarations with `let` and `const`

Declare variable in modern JS (ES6 and later):
* `let` keyword: e.g. `let i;`
* multi-variables supported: e.g. `let i, sum;`
* **Initialized** variable is recommanded: e.g. `let message = "hello";`
  * Uninitilized variable's value is `undefined` until assigned with value.

Declare a constant using `const`:
* constant variable need initilization at the same time
* Their value cannot be changed ever, otherwise, cause `TypeError`
* Convention: constant are **capital letters**: e.g. `HTTP_NOT_FOUND`

```js
const H0 = 74;         // Hubble constant (km/s/Mpc)
const C = 299792.458;  // Speed of light in a vacuum (km/s)
const AU = 1.496E8;    // Astronomical Unit: distance to the sun (km)
```

#### Variable and constant scope

Block scope (Same as in C/C++)
* Variable declared outside code blocks are **global variables**

#### Repeated declarations

* Declare multiple vars in a same scope with same name is **syntax error**
  * It's not when it's in nested scope

```js
const x = 1;        // Declare x as a global constant
if (x === 1) {
    let x = 2;      // Inside a block x can refer to a different value
    console.log(x); // Prints 2
}
console.log(x);     // Prints 1: we're back in the global scope now
let x = 3;          // ERROR! Syntax error trying to re-declare x
```

#### Declarations and types

* In statically typed language (e.g. C/C++ or Java), variable declaration is to specify type of value
* No type associated with JS declaration. A JS variable can hold any type

```js
let i = 10;
i = "ten";
```

### 3.10.2 Variable Declarations with `var`

Before ES6
* Declare a variable using `var`
* No constant declaration

`var` syntax:
* similar as `let`

```js
var x;
var data = [], count = data.length;
for(var i = 0; i < count; i++) console.log(data[i]);
```

`var` vs. `let`:
* Scope of `var`: **Function scope**
  * variables declared with `var` do not have (nested) block scope. The only scope they have is the function that containing them.
* Global variable using `var`: 
  * if `var` used outside of a function body, it declare a global variable
  * Global variable declared with `var` are implemented as properties of the global object, which can be referenced using `globalThis`. e.g.
    * in function, declare `var x = 2`; from program's view, it's same as `globalThis.x = 2`
    * variable declared by `let` & `const` are not global object property
* `var` allows redeclare, due to function scope (instead of block scope)
* variable decalred with `var`, its declaration is lifted up (**hosted**) to top of enclosing function
* Refer to [Stack Overflow: What's the difference-between-using-let-and-var](https://stackoverflow.com/questions/762011/whats-the-difference-between-using-let-and-var)

### 3.10.3 Destructuring Assignment

**Destructuring Assignment**:
* Compound declaration and assignment syntax
* value on RHS of `=` is an array or object (a "structured" value)
* LHS specify variable names to mimic array/object.

```js
let [x,y] = [1,2];  // Same as let x=1, y=2
[x,y] = [x+1,y+1];  // Same as x = x + 1, y = y + 1
[x,y] = [y,x];      // Swap the value of the two variables
[x,y]               // => [3,2]: the incremented and swapped values
```

Destructuring Assignment can catch return value (arrays) from function
```js
// Convert [x,y] coordinates to [r,theta] polar coordinates
function toPolar(x, y) {
    return [Math.sqrt(x*x+y*y), Math.atan2(y,x)];
}

// Convert polar to Cartesian coordinates
function toCartesian(r, theta) {
    return [r*Math.cos(theta), r*Math.sin(theta)];
}

let [r,theta] = toPolar(1.0, 1.0);  // r == Math.sqrt(2); theta == Math.PI/4
let [x,y] = toCartesian(r,theta);   // [x, y] == [1.0, 1,0]
```

Use destructuring assignment to loop over name/value pairs
```js
let o = { x: 1, y: 2 }; // The object we'll loop over
for(const [name, value] of Object.entries(o)) {
    console.log(name, value); // Prints "x 1" and "y 2"
}
```

#### Matching number of variables

Number of variables on LHS no need to match number of array element RHS:
* Use extra commas `,` to skip certain values
```js
let [x,y] = [1];     // x == 1; y == undefined
[x,y] = [1,2,3];     // x == 1; y == 2
[,x,,y] = [1,2,3,4]; // x == 2; y == 4
```

* Use three dots `...` before last variable name on LHS, to collect all unused or remaining values into a single variable:
```js
let [x, ...y] = [1,2,3,4];  // y == [2,3,4]
```

#### More general: iterable object on RHS

Destructuring assignment do not need array, an **iterable object** is enough
```js
let [first, ...rest] = "Hello"; // first == "H"; rest == ["e","l","l","o"]
```

#### More general: object value on RHS

Destructuring assignment can be performed when RHS is an object value.
* Syntax on LHS: a comma-separated list of variable names within curly braces

```js
let transparent = {r: 0.0, g: 0.0, b: 0.0, a: 1.0}; // A RGBA color
let {r, g, b} = transparent;  // r == 0.0; g == 0.0; b == 0.0
```

#### More general: destruct object into individual variables

Using destructuring assignment to get individual properties from object
```js
// Same as const sin=Math.sin, cos=Math.cos, tan=Math.tan
const {sin, cos, tan} = Math;
```
Note:
* `Math` object has many properties.
* Those that are not named are ignored.
* If LHS included variable name which is not a property of `Math`, that variable will be `undefined`

Name convention: 
* not required to use variable names that match property names of the object.
* Syntax: **colon-separated pairs of identifiers**
  * 1st is the name of property whose value is to be assigned
  * 2nd is the name of variable to assign to
* This usage generate more complex
```js
// Same as const cosine = Math.cos, tangent = Math.tan;
const { cos: cosine, tan: tangent } = Math;
```

### More general: nested objects

```js
let points = [{x: 1, y: 2}, {x: 3, y: 4}];     // An array of two point objects
let [{x: x1, y: y1}, {x: x2, y: y2}] = points; // destructured into 4 variables.
(x1 === 1 && y1 === 2 && x2 === 3 && y2 === 4) // => true
```